@@ -1,4 +1,3 @@
-{-# LANGUAGE DerivingStrategies #-}
 {-# LANGUAGE DerivingVia #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE LambdaCase #-}
@@ -16,6 +15,11 @@ Description : Conversion from DuckDB column values to Haskell types.
 module Database.DuckDB.Simple.FromField (
     Field (..),
     FieldValue (..),
+    StructField (..),
+    StructValue (..),
+    UnionMemberType (..),
+    UnionValue (..),
+    LogicalTypeRep (..),
     BitString(..),
     bsFromBool,
     BigNum (..),
@@ -33,6 +37,7 @@ module Database.DuckDB.Simple.FromField (
 import Control.Exception (Exception, SomeException (..))
 import Data.Bits (Bits (..), finiteBitSize)
 import qualified Data.ByteString as BS
+import Data.Array (Array, (!), bounds, listArray, range, assocs)
 import Data.Data (Typeable, typeRep)
 import Data.Int (Int16, Int32, Int64, Int8)
 import Data.Map.Strict (Map)
@@ -57,6 +62,13 @@ import Database.DuckDB.Simple.Types (Null (..))
 import GHC.Num.Integer (integerFromWordList)
 import Numeric.Natural (Natural)
 import qualified Data.UUID as UUID
+import Database.DuckDB.Simple.LogicalRep (
+    LogicalTypeRep (..),
+    StructField (..),
+    StructValue (..),
+    UnionMemberType (..),
+    UnionValue (..)
+ )
 
 -- | Internal representation of a column value.
 data FieldValue
@@ -87,8 +99,11 @@ data FieldValue
     | FieldBit BitString
     | FieldBigNum BigNum
     | FieldEnum Word32
+    | FieldArray (Array Int FieldValue)
     | FieldList [FieldValue]
     | FieldMap [(FieldValue, FieldValue)]
+    | FieldStruct (StructValue FieldValue)
+    | FieldUnion (UnionValue FieldValue)
     deriving (Eq, Show)
 
 data DecimalValue = DecimalValue
@@ -279,6 +294,20 @@ class FromField a where
 instance FromField FieldValue where
     fromField Field{fieldValue} = Ok fieldValue
 
+instance FromField (StructValue FieldValue) where
+    fromField f@Field{fieldValue} =
+        case fieldValue of
+            FieldStruct structVal -> Ok structVal
+            FieldNull -> returnError UnexpectedNull f ""
+            _ -> returnError Incompatible f "expected STRUCT"
+
+instance FromField (UnionValue FieldValue) where
+    fromField f@Field{fieldValue} =
+        case fieldValue of
+            FieldUnion unionVal -> Ok unionVal
+            FieldNull -> returnError UnexpectedNull f ""
+            _ -> returnError Incompatible f "expected UNION"
+
 instance FromField Null where
     fromField f@Field{fieldValue} =
         case fieldValue of
@@ -528,6 +557,8 @@ instance {-# OVERLAPPABLE #-} (Typeable a, FromField a) => FromField [a] where
         case fieldValue of
             FieldList entries ->
                 traverse (uncurry parseElement) (zip [0 :: Int ..] entries)
+            FieldArray arr ->
+                traverse (uncurry parseElement) (assocs arr)
             FieldNull -> returnError UnexpectedNull f ""
             _ -> returnError Incompatible f ""
       where
@@ -543,6 +574,26 @@ instance {-# OVERLAPPABLE #-} (Typeable a, FromField a) => FromField [a] where
                 idxText = Text.pack (show idx)
              in base <> Text.pack "[" <> idxText <> Text.pack "]"
 
+instance (Typeable a, FromField a) => FromField (Array Int a) where
+    fromField f@Field{fieldValue} =
+        case fieldValue of
+            FieldArray arr -> do
+                converted <- traverse (traverseElement arr) (range (bounds arr))
+                pure (listArray (bounds arr) converted)
+            FieldNull -> returnError UnexpectedNull f ""
+            _ -> returnError Incompatible f ""
+      where
+        traverseElement arr idx =
+            fromField
+                Field
+                    { fieldName = fieldNameWithIndex idx
+                    , fieldIndex = fieldIndex f
+                    , fieldValue = arr ! idx
+                    }
+        fieldNameWithIndex idx =
+            let idxText = Text.pack (show idx)
+             in fieldName f <> Text.pack "[" <> idxText <> Text.pack "]"
+
 instance (Ord k, Typeable k, Typeable v, FromField k, FromField v) => FromField (Map k v) where
     fromField f@Field{fieldValue} =
         case fieldValue of
@@ -692,6 +743,9 @@ fieldValueTypeName = \case
     FieldBit{} -> "BIT"
     FieldBigNum{} -> "BIGNUM"
     FieldEnum{} -> "ENUM"
+    FieldArray{} -> "ARRAY"
     FieldList{} -> "LIST"
     FieldMap{} -> "MAP"
-    FieldUUID{} -> "UUID"
\ No newline at end of file
+    FieldUUID{} -> "UUID"
+    FieldStruct{} -> "STRUCT"
+    FieldUnion{} -> "UNION"
